// TFont.java

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

class TFont extends JFrame {
    DisplayPanel displayPanel;
    JComboBox fontsBox, fontStylesBox, fontSizesBox;

    String[] fontStyleLabels = {"Plain", "Bold", "Italic", "Bold&Italic"};
    int BOLDITALIC = Font.BOLD|Font.ITALIC;
    int[] fontStyles = {Font.PLAIN, Font.BOLD, Font.ITALIC, BOLDITALIC};
    String[] fontSizeLabels = {"8", "9", "10", "11", "12", 
                               "14", "18", "25", "36", "72"};

    public TFont() {
        // 1. Assign a title to the frame and get a handle on
        // the frame's content pane
        super("TFont");
        Container container = getContentPane();

        // 2. Create a display panel and add it to the frame 
        displayPanel = new DisplayPanel();
        container.add(displayPanel);

        // 3. Create a control panel with titled border
        JPanel controlPanel = new JPanel();
        controlPanel.setLayout(new GridLayout(1,3));
        TitledBorder border = new TitledBorder(
            "Select a Font, Font Style, and Font Size...");
        controlPanel.setBorder(border);

        // 4. Create combo boxes for font names, styles, and sizes
        fontsBox = new JComboBox(displayPanel.fontFamilyNames);
        fontsBox.setSelectedItem("Arial"); // default selection
        fontsBox.addActionListener(new ComboBoxListener());

        fontStylesBox = new JComboBox(fontStyleLabels);
        fontStylesBox.addActionListener(new ComboBoxListener());

        fontSizesBox = new JComboBox(fontSizeLabels);
        fontSizesBox.setSelectedItem("36");
        fontSizesBox.addActionListener(new ComboBoxListener());

        // 5. Add the combo boxes to the control panel and add the
        // panel to the main frame.
        controlPanel.add(fontsBox);
        controlPanel.add(fontStylesBox);
        controlPanel.add(fontSizesBox);
        container.add(BorderLayout.SOUTH, controlPanel);

        // 6. Add a frame closing listener and display the frame
        addWindowListener(new WindowEventHandler());
        pack();
        setSize(400, 250);
        show(); // Display the frame
    }

    // 7. The main method...
    public static void main(String arg[]) {
        new TFont();
    }

    // 8. Class to handle closing of the frame
    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }

    // 9. Combo box listener to handle font name, style and size
    // selections in the respective combo boxes.
    class ComboBoxListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JComboBox tempBox = (JComboBox) e.getSource();

            if (tempBox.equals(fontsBox)) {
                displayPanel.fontFamilyName =
                             (String) tempBox.getSelectedItem();
                displayPanel.repaint();
            }
            else if (tempBox.equals(fontStylesBox)) {
                displayPanel.fontStyle =
                             fontStyles[tempBox.getSelectedIndex()];
                displayPanel.repaint();
            }
            else if (tempBox.equals(fontSizesBox)) {
                displayPanel.fontSize =
                             Integer.parseInt((String) tempBox.getSelectedItem());
                displayPanel.repaint();
            }
        }
    }

    // 10. Definition of display panel
    class DisplayPanel extends JPanel {
        String displayText;
        Font currentFont;
        String fontFamilyName;
        int fontStyle;
        int fontSize;

        GraphicsEnvironment ge;
        String[] fontFamilyNames;
        
        // 11. Constructor
        public DisplayPanel() {
            // Logo to be displayed on the panel
            displayText = "Java 2D Fonts";
            fontFamilyName = "Arial";
            fontStyle = Font.PLAIN;
            fontSize = 36;
            
            // Retrieve all the font family names from the platform
            System.out.println("Loading Fonts... Please Wait...");
            ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
            fontFamilyNames = ge.getAvailableFontFamilyNames();

            setBackground(Color.white);  // For canvas background color
            setSize(400, 225);  // Canvas width and height
        }

        // 12. The update method...
        public void update(Graphics g) {
            g.clearRect(0, 0, getWidth(), getHeight());
            paintComponent(g);
        }

        // 13. The painting method...
        public void paintComponent(Graphics g) {
            super.paintComponent(g);

            // 14. Create the graphics context object
            Graphics2D g2D = (Graphics2D) g;

            // 15. Prepare the current font and apply it to
            // the display text.
            currentFont = new Font(fontFamilyName,
                                   fontStyle,
                                   fontSize);
            g2D.setFont(currentFont);
            g2D.drawString(displayText, 25, 100);
        }
    }
}
