// TBufferedImage2.java

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

class TBufferedImage2 extends JFrame {
    DisplayCanvas canvas;
    
    public TBufferedImage2() {
        // 1. Assign a title to the frame and get a handle on
        // the frame's content pane
        super("TBufferedImage2");
        Container container = getContentPane();

        // 2. Add the canvas
        canvas = new DisplayCanvas();
        TitledBorder border = new TitledBorder(
            "Drag the Butterfly Using the Mouse");
        border.setTitlePosition(TitledBorder.BOTTOM);
        canvas.setBorder(border);
        container.add(canvas);

        //  3. Add a frame closing listener and display the frame
        addWindowListener(new WindowEventHandler());
        setSize(450, 400);
        show(); // Display the frame
    }

    // 4. Code to handle closing of the frame
    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }

    // 5. The main method...
    public static void main(String arg[]) {
        new TBufferedImage2();
    }
}

// 6. Canvas to display the butterfly
class DisplayCanvas extends JPanel {
    int x, y; // Location of image.
    BufferedImage bi;

    // Constructor
    DisplayCanvas() {
        setBackground(Color.white);  // For canvas background color
        setSize(450, 400);  // Canvas width=450 height=400
        addMouseMotionListener(new MouseMotionHandler());

        Image image = getToolkit().getImage("butterfly.gif");

        MediaTracker mt = new MediaTracker(this);
        mt.addImage(image, 1);
        try {
            mt.waitForAll();
        } catch (Exception e) {
            System.out.println("Exception while loading image.");
        }

        // If the image has an unknown width, the image is not created
        // by using the specified file. Therefore exit the program!  
        if (image.getWidth(this) == -1) {
            System.out.println("*** Make sure you have the image "
                + "(butterfly.gif) file in the same directory.***");
            System.exit(0);
        }

        // NOTE: You need to use the image type ARGB to consider the
        //       effect of alpha value of an image.
        bi = new BufferedImage(image.getWidth(this),
                               image.getHeight(this),
                               BufferedImage.TYPE_INT_ARGB);
        Graphics2D big = bi.createGraphics();
        big.drawImage(image, 0, 0, this);
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        // Create the graphics context object
        Graphics2D g2D = (Graphics2D) g;

        g2D.drawImage(bi, x, y, this);
    }

    // 7. Mouse listener class
    class MouseMotionHandler extends MouseMotionAdapter {
        public void mouseDragged(MouseEvent e) {
            x = e.getX(); y = e.getY();
            repaint();
        }
    }
}
