// TBufferedImage3.java

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;

class TBufferedImage3 extends JFrame {
    AnimationCanvas canvas;
    JButton startButton, stopButton;
    
    public TBufferedImage3() {
        // 1. Assign a title to the frame and get a handle on
        // the frame's content pane
        super("TBufferedImage3");
        Container container = getContentPane();

        // 2. Add the canvas with rectangles
        canvas = new AnimationCanvas();
        container.add(canvas);

        // 3. Add buttons to start or stop the animation
        startButton = new JButton("Start Animation");
        startButton.addActionListener(new ButtonListener());
        stopButton = new JButton("Stop Animation");
        stopButton.addActionListener(new ButtonListener());
        JPanel panel = new JPanel();
        panel.add(startButton); panel.add(stopButton);
        container.add(BorderLayout.SOUTH, panel);

        //  4. Add a frame closing listener and display the frame
        addWindowListener(new WindowEventHandler());
        setSize(450, 425);
        show(); // Display the frame
    }

    // 5. Code to handle closing of the frame
    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }

    // 6. Button listener
    class ButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JButton temp = (JButton) e.getSource();

            if (temp.equals(startButton)) {
                canvas.start();
            }
            else if (temp.equals(stopButton)) {
                canvas.stop();
            }
        }
    }

    // 7. The main method...
    public static void main(String arg[]) {
        new TBufferedImage3();
    }
}

// 8. Definition of the canvas that displays animation.
class AnimationCanvas extends JPanel implements Runnable {
    Thread thread;

    Image image;
    BufferedImage bi;

    double x, y, xi, yi; 
    int rotate;
    double scale; int UP = 0; int DOWN = 1;
    int scaleDirection;

    // 9. Constructor
    AnimationCanvas() {
        setBackground(Color.green);  // For canvas background color
        setSize(450, 400);  // Canvas width=450 height=400

        image = getToolkit().getImage("butterfly.gif");

        MediaTracker mt = new MediaTracker(this);
        mt.addImage(image, 1);
        try {
            mt.waitForAll();
        } catch (Exception e) {
            System.out.println("Exception while loading image.");
        }

        // If the image has an unknown width, the image is not created
        // by using the specified file. Therefore exit the program.  
        if (image.getWidth(this) == -1) {
            System.out.println("*** Make sure you have the image "
                + "(butterfly.gif) file in the same directory.***");
            System.exit(0);
        }

        rotate = (int) (Math.random() * 360);
        scale = Math.random() * 1.5;
        scaleDirection = DOWN;

        xi = 50.0; yi = 50.0;

    }

    // This method computes the step size for animation.
    public void step(int w, int h) {
        // upgrade the translation coordinates
        x += xi; y += yi;

        // the x and y exceed the dimensions of canvas
        if (x > w) {
            x = w - 1;
            xi = Math.random() * -w/32;
        }
        if (x < 0) {
            x = 2;
            xi =  Math.random() * w/32;
        }
        if (y > h) {
            y = h - 2;
            yi = Math.random() * -h/32;
        }
        if (y < 0) {
            y = 2;
            yi = Math.random() * h/32;
        }

        // upgrade the rotation coordinates
        if ((rotate += 5) == 360) {
            rotate = 0;
        }
        // upgrade the scaling coordinates depending on the
        // increase or decrease in size. If the increase in size
        // exceeds a limit of 1.5, decrease the size. If the
        // decrease in size falls below 0.5, increase the size.
        if (scaleDirection == UP) {
            if ((scale += 0.5) > 1.5) {
                scaleDirection = DOWN;
            }
        }
        // upgrade the scaling coordinates
        else if (scaleDirection == DOWN) {
            if ((scale -= .05) < 0.5) {
                scaleDirection = UP;
            }
        }
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Dimension d = getSize();

        bi = new BufferedImage(d.width, d.height, BufferedImage.TYPE_INT_ARGB);
        Graphics2D big =  bi.createGraphics();

        step(d.width, d.height);

        AffineTransform at = new AffineTransform();
        at.setToIdentity();
        at.translate(x, y);
        at.rotate(Math.toRadians(rotate));
        at.scale(scale, scale);
        big.drawImage(image, at, this);

        Graphics2D g2D = (Graphics2D) g;
        g2D.drawImage(bi, 0 , 0, null);

        big.dispose();
    }

    // Starts the thread
    public void start() {
        thread = new Thread(this);
        thread.setPriority(Thread.MIN_PRIORITY);
        thread.start();
    }

    // Stops the thread
    public void stop() {
        if (thread != null)
            thread.interrupt();
        thread = null;
    }

    // Runs the thread
    public void run() {
        Thread me = Thread.currentThread();
        while (thread == me) {
            repaint();
        }
        thread = null;
    }
}
