// TLookupOp.java

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

class TLookupOp extends JFrame {
    DisplayPanel displayPanel;
    JButton brightenButton, darkenButton,
            contrastIncButton, contrastDecButton,
            reverseButton, resetButton;
    
    public TLookupOp() {
        // 1. Assign a title to the frame and get a handle on
        // the frame's content pane
        super("TLookupOp");
        Container container = getContentPane();

        // 2. Create and add a panel to displays images.
        displayPanel = new DisplayPanel();
        container.add(displayPanel);

        // 3. Create a control panel and add it to the main frame.
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(3, 2));
        panel.setBorder(new TitledBorder(
            "Click a Button to Perform the Associated Operation and Reset..."));

        brightenButton = new JButton("Brightness >>");
        brightenButton.addActionListener(new ButtonListener());
        darkenButton = new JButton("Darkness >>");
        darkenButton.addActionListener(new ButtonListener());
        contrastIncButton = new JButton("Contrast >>");
        contrastIncButton.addActionListener(new ButtonListener());
        contrastDecButton = new JButton("Contrast <<");
        contrastDecButton.addActionListener(new ButtonListener());
        reverseButton = new JButton("Negative");
        reverseButton.addActionListener(new ButtonListener());
        resetButton = new JButton("Reset");
        resetButton.addActionListener(new ButtonListener());

        panel.add(brightenButton);
        panel.add(darkenButton);
        panel.add(contrastIncButton);
        panel.add(contrastDecButton);
        panel.add(reverseButton);
        panel.add(resetButton);

        container.add(BorderLayout.SOUTH, panel);

        // 4. Add a frame closing listener and display the frame
        addWindowListener(new WindowEventHandler());
        // Assign suitable size. 25 is for the control panel
        setSize(displayPanel.getWidth(), displayPanel.getHeight() + 25);
        show(); // Display the frame
    }

    // 5. Code to handle closing of the frame
    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }

    // 6. The main method...
    public static void main(String arg[]) {
        new TLookupOp();
    }

    
    // 7. Action listener to handle the event fired by buttons.
    class ButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JButton button = (JButton) e.getSource();

            if (button.equals(brightenButton)) {
                displayPanel.brightenLUT();
                displayPanel.applyFilter();
                displayPanel.repaint();
            }
            else if (button.equals(darkenButton)) {
                displayPanel.darkenLUT();
                displayPanel.applyFilter();
                displayPanel.repaint();
            }
            else if (button.equals(contrastIncButton)) {
                displayPanel.contrastIncLUT();
                displayPanel.applyFilter();
                displayPanel.repaint();
            }
            else if (button.equals(contrastDecButton)) {
                displayPanel.contrastDecLUT();
                displayPanel.applyFilter();
                displayPanel.repaint();
            }
            else if (button.equals(reverseButton)) {
                displayPanel.reverseLUT();
                displayPanel.applyFilter();
                displayPanel.repaint();
            }
            else if (button.equals(resetButton)) {
                displayPanel.reset();
                displayPanel.repaint();
            }
        }
    } 
}

// 8. Definition of the display panel.
class DisplayPanel extends JPanel {
    Image displayImage;
    BufferedImage bi;
    Graphics2D big;
    LookupTable LUT;

    // Constructor
    DisplayPanel() {
        setBackground(Color.black);  // panel background color
        loadImage();
        setSize(displayImage.getWidth(this),
                displayImage.getWidth(this));  // panel width and height

        createBufferedImage();
    }

    // This method loads the specified image.
    public void loadImage() {
        // Create an image object using the specified file
        displayImage = Toolkit.getDefaultToolkit().getImage("images/flowers1.jpg");
        MediaTracker mt = new MediaTracker(this);
        mt.addImage(displayImage, 1);
        try {
            mt.waitForAll();
        } catch (Exception e) {
            System.out.println("Exception while loading.");
        }

        // If the image has an unknown width, the image is not created
        // by using the suggested file. Therefore, exit the program.  
        if (displayImage.getWidth(this) == -1) {
            System.out.println("*** Make sure you have the suggested image "
                            + "(*.jpg) file in the same directory.***");
            System.exit(0);
        }
    }

    // This method creates a buffered image. 
    public void createBufferedImage() {
        // NOTE: Do not use createImage() method here. As the
        // component paint() method is not at called, the createImage()
        // returns a null object.
        bi = new BufferedImage(displayImage.getWidth(this),
                               displayImage.getHeight(this),
                               BufferedImage.TYPE_INT_ARGB);

        // Draw the texture image into the memory buffer.
        big = bi.createGraphics();
        big.drawImage(displayImage, 0, 0, this);
    }

    // Create a look-up table for brightening pixels.
    public void brightenLUT() {
        short brighten[] = new short[256];
        for (int i=0; i<256; i++) {
            short pixelValue = (short) (i+10); 
            if (pixelValue > 255)
                pixelValue = 255;
            else if (pixelValue < 0)
                pixelValue = 0;
            brighten[i] =  pixelValue;
        }
        LUT = new ShortLookupTable(0, brighten);
    }

    // Create a look-up table for darkening pixels.
    public void darkenLUT() {
        short brighten[] = new short[256];
        for (int i=0; i<256; i++) {
            short pixelValue = (short) (i-10);
            if (pixelValue > 255)
                pixelValue = 255;
            else if (pixelValue < 0)
                pixelValue = 0;
            brighten[i] =  pixelValue;
        }
        LUT = new ShortLookupTable(0, brighten);
    }

    // Create a look-up table to increase the contrast.
    public void contrastIncLUT() {
        short brighten[] = new short[256];
        for (int i=0; i<256; i++) {
            short pixelValue = (short) (i*1.2);
            if (pixelValue > 255)
                pixelValue = 255;
            else if (pixelValue < 0)
                pixelValue = 0;
            brighten[i] =  pixelValue;
        }
        LUT = new ShortLookupTable(0, brighten);
    }

    // Create a look-up table to decrease the contrast.
    public void contrastDecLUT() {
        short brighten[] = new short[256];
        for (int i=0; i<256; i++) {
            short pixelValue = (short) (i/1.2);
            if (pixelValue > 255)
                pixelValue = 255;
            else if (pixelValue < 0)
                pixelValue = 0;
            brighten[i] =  pixelValue;
        }
        LUT = new ShortLookupTable(0, brighten);
    }

    // Create a look-up table to create the nagative of an image.
    public void reverseLUT() {
        byte reverse[] = new byte[256];
        for (int i=0; i<256; i++) {
            reverse[i] = (byte) (255-i);
        }
        LUT = new ByteLookupTable(0, reverse);
    }

    // Reset the display panel
    public void reset() {
        big.setColor(Color.black);
        big.clearRect(0, 0, bi.getWidth(this), bi.getHeight(this));
        big.drawImage(displayImage, 0, 0, this);
    }

    // This method creates a look-up filter and applies it to
    // the buffered image.
    public void applyFilter() {
        LookupOp lop = new LookupOp(LUT, null);
        lop.filter(bi, bi);
    }

    // The update method
    public void update(Graphics g) {
        g.clearRect(0, 0, getWidth(), getHeight());
        paintComponent(g);
    }

    // The paintComponent method.
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        // Create the graphics context object
        Graphics2D g2D = (Graphics2D) g;

        // Draw the buffered Image on to the screen
        g2D.drawImage(bi, 0,0, this);
    }
}
