// TPrinting2.java

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.print.*;
import java.util.Vector;

class TPrinting2 extends JFrame {
    DrawingCanvas canvas;
    JRadioButton portraitButton, landscapeButton, rLandscapeButton;
    JButton addButton, printButton, clearButton;

    public TPrinting2() {
        // 1. Assign a name to the frame and retrieve its content pane
        super("TPrinting2");
        Container container = getContentPane();

        // 2. Add the drawing canvas
        canvas = new DrawingCanvas();
        container.add(canvas);

        // 3. Create and add a control panel
        JPanel panel = new JPanel();
        panel.setBorder(new TitledBorder(
            "Select Suitable Format, "
           +"Add the Drawing Pages to Book and Print..."));
        // Add the panel to the main frame
        container.add(panel, BorderLayout.SOUTH);


        // 4. Assign grid layout to the panel
        panel.setLayout(new GridLayout(2,6));

        // 5. Create radio buttons to select a format
        portraitButton = new JRadioButton("Portrait", true);
        landscapeButton = new JRadioButton("Landscape");
        rLandscapeButton = new JRadioButton("Reverse Landscape");

        // Prepare the radio buttons as a mutually exclusive group
        ButtonGroup exclusiveGroup = new ButtonGroup();
        exclusiveGroup.add(portraitButton);
        exclusiveGroup.add(landscapeButton);
        exclusiveGroup.add(rLandscapeButton);

        // 6. Register an action listener with radio buttons
        ActionListener buttonListener = new ButtonListener();
        portraitButton.addActionListener(buttonListener);
        landscapeButton.addActionListener(buttonListener);
        rLandscapeButton.addActionListener(buttonListener);

        // 7. Add the radio buttons to the panel
        panel.add(portraitButton);
        panel.add(landscapeButton);
        panel.add(rLandscapeButton);

        // 8. Create push buttons and add them to the panel
        addButton = new JButton("Add to Book");
        printButton = new JButton("Print");
        clearButton = new JButton("Clear");

        // 9. Register the action listener with the push buttons
        addButton.addActionListener(buttonListener);
        printButton.addActionListener(buttonListener);
        clearButton.addActionListener(buttonListener);

        // 10. Add the push buttons to the panel
        panel.add(addButton);
        panel.add(printButton);
        panel.add(clearButton);

       
        // 11. Add a frame closing listener and display the frame
        addWindowListener(new WindowEventHandler());
        setSize(450, 425); // Frame size
        show(); // Display the frame
    }

    // 12. Code to handle closing of the frame
    class WindowEventHandler extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        }
    }

    // 13. The main method...
    public static void main(String arg[]) {
        new TPrinting2();
    }


    // 14. Definition of button listener class
    class ButtonListener implements ActionListener {
        PrinterJob jobControl;  // print job control object
        PageFormat pageFormat;
        Book book;  // A reference to multi-page document

        ButtonListener() {
            jobControl = PrinterJob.getPrinterJob();
            pageFormat = jobControl.defaultPage();
            book = new Book();
        }

        // 15. Handling actions method...
        public void actionPerformed(ActionEvent e) {
            // Retrieve the source object that fires action events
            Object obj = e.getSource();

            // When the source objects are radio buttons, assign
            // relevant print formats.
            if (obj instanceof JRadioButton) {
                JRadioButton tempButton = (JRadioButton) obj;

                if (tempButton.equals(portraitButton)) {
                    pageFormat.setOrientation(PageFormat.PORTRAIT);
                }

                else if (tempButton.equals(landscapeButton)) { 
                    pageFormat.setOrientation(PageFormat.LANDSCAPE);
                }

                // NOTE: Believed to be a bug in versions
                // JDK 1.2 --> Java 2 SDK 1.2.2. Tested on Windows 95.
                // The reverse landscape does not function. 
                else if (tempButton.equals(rLandscapeButton)) {
                    pageFormat.setOrientation(PageFormat.REVERSE_LANDSCAPE);
                }
            }

            // When the source objects are push-type buttons, execute
            // the relevant operation.
            else if (obj instanceof JButton) {
                JButton tempButton = (JButton) obj;

                if (tempButton.equals(addButton)) {
                    // Append a page to the book with the selected format
                    book.append(
                        new PrintableCanvas(canvas.getShapesVector()),
                        (PageFormat) pageFormat.clone());
                }
                else if (tempButton.equals(printButton)) {
                   // Assign book to the job control system
                   jobControl.setPageable(book);

                   // If the OK button is clicked on print dialog, continue printing
                   if (jobControl.printDialog()) {
                       try {
                           jobControl.print();
                       } catch (Exception pe) {
                           System.out.println("Printing Exception Occured!");
                           pe.printStackTrace();
                       }
                   }
               }
             
               // To clear the canvas
               else if (tempButton.equals(clearButton)) {
                   canvas.shapesVector.clear();
                   canvas.clear = true;
                   canvas.repaint();
               }
           }
        }
    }


    // 16. Canvas for drawing 
    class DrawingCanvas extends Canvas {
        private Vector shapesVector;
        Point2D currPoint, newPoint;
        Line2D line;
        boolean clear = false;

        // 17. Constructor
        DrawingCanvas() {
            // Initialize the shapes vector
            shapesVector = new Vector();

            // Register the mouse and mouse-motion listeners
            addMouseListener(new MouseHandler());
            addMouseMotionListener(new MouseMotionHandler());

            setBackground(Color.white);
            setSize(450, 400);  // canvas width & height
        }

        // 18. Retrieves the shapes vector
        public Vector getShapesVector() {
            return shapesVector;
        }

        // 19. The update() method...
        public void update(Graphics g) {
            // Set up the 2D graphics context
            Graphics2D g2D = (Graphics2D) g;

            g2D.setColor(Color.black);

            // Draw the lines on the canvas
            if (currPoint != null && newPoint != null) {
                line = new Line2D.Float(currPoint, newPoint);
                g2D.draw(line);

                // Store the lines in the shapes vector.
                shapesVector.addElement(line);
            }
            currPoint = newPoint;  // update the current point

            // Clear the background when the Clear button is clicked.
            if (clear) {
                g2D.setColor(Color.white);
                g2D.fillRect(0, 0, getWidth(), getHeight());
                clear = false;
            }
        }

        // 20. The paint() method...
        public void paint(Graphics g) {
            // Set up the 2D graphics context
            Graphics2D g2D = (Graphics2D) g;

            g2D.setColor(Color.black);

            // Draw the lines whenever the canvas is resized.
            for (int i=0; i<shapesVector.size(); i++) {
                Line2D.Float line2D = (Line2D.Float) shapesVector.elementAt(i);
                g2D.draw(line2D);
            } 
        }

        // 21. Mouse handler
        class MouseHandler extends MouseAdapter {
            public void mousePressed(MouseEvent e) {
                currPoint = newPoint = e.getPoint();
                repaint();
            }
        }

        // 22. Mouse motion handler
        class MouseMotionHandler extends MouseMotionAdapter {
            public void mouseDragged(MouseEvent e) {
                newPoint = e.getPoint();
                repaint();
            }
        }
    }

    // 23. Printable canvas class
    class PrintableCanvas implements Printable {
        // Stores the line objects drawn on the drawing canvas.
        private Vector linesVector; 

        // Constructor: Pass the lines (shapes) drawn over the
        // drawing canvas.
        public PrintableCanvas(Vector vector) {
            linesVector = (Vector) vector.clone();
        }

        // The print() method...
        public int print(Graphics pg, PageFormat pf, int pi)
                                     throws PrinterException {
            // Set up the 2D printer graphics context
            Graphics2D pg2D = (Graphics2D) pg;

            // Translate the origin of printer graphics context
            // to the origin of imageable area.
            pg2D.translate(pf.getImageableX(), pf.getImageableY());

            // Bug Alert!: The versions of JDK1.2 or Java 2 SDK 1.2.2 are
            // believed to have a bug. Tested on Windows 95
            // Bug Description: Assign a color to the printer context
            // and draw something to it to be printed on the first page,
            // and then assign the same color once again and
            // draw something to be printed on the second page. Then
            // the system does not print anything on the second page (and
            // and remaining pages, if any).

            // Solution: Hack the print system!!!
            // Assing an arbitrary color, then you need to draw something.
            pg2D.setPaint(Color.green);
            pg2D.drawString(" ", 100, 100);// Draw some invisible thing! 

            // Now the regular code comes here; this code assigns black color
            // to the context, and then prints the lines stored in the
            // lines vector.
            pg2D.setPaint(Color.black);
            for (int i=0; i<linesVector.size(); i++) {
                Line2D.Float line2D = (Line2D.Float) linesVector.elementAt(i);
                pg2D.draw(line2D);
            }

            // On successfully printing the page
            return Printable.PAGE_EXISTS;
        }
    }
}
